@@ -1,3 +1,4 @@
+import { describe, it, expect } from "vitest";
 import * as util from "util";
 import * as pathToRegexp from "./index";
 import semver from "semver";
@@ -11,10 +12,10 @@ type Test = [
       string,
       (string | undefined)[] | null,
       pathToRegexp.Match?,
-      pathToRegexp.RegexpToFunctionOptions?
+      pathToRegexp.RegexpToFunctionOptions?,
     ]
   >,
-  Array<[any, string | null, pathToRegexp.TokensToFunctionOptions?]>
+  Array<[any, string | null, pathToRegexp.TokensToFunctionOptions?]>,
 ];
 
 /**
@@ -1352,7 +1353,7 @@ const TESTS: Test[] = [
         prefix: ".",
         suffix: "",
         modifier: "+",
-        pattern: "[^\\/#\\?]+?",
+        pattern: "(?:(?!\\.)[^\\/#\\?])+?",
       },
     ],
     [
@@ -1396,7 +1397,7 @@ const TESTS: Test[] = [
         prefix: ".",
         suffix: "",
         modifier: "",
-        pattern: "[^\\/#\\?]+?",
+        pattern: "(?:(?!\\.)[^\\/#\\?])+?",
       },
       ".",
     ],
@@ -1429,13 +1430,13 @@ const TESTS: Test[] = [
         prefix: ".",
         suffix: "",
         modifier: "",
-        pattern: "[^\\/#\\?]+?",
+        pattern: "(?:(?!\\.)[^\\/#\\?])+?",
       },
     ],
     [
       ["/route.html", ["/route.html", "route", "html"]],
       ["/route", null],
-      ["/route.html.json", ["/route.html.json", "route", "html.json"]],
+      ["/route.html.json", ["/route.html.json", "route.html", "json"]],
     ],
     [
       [{}, null],
@@ -1458,13 +1459,13 @@ const TESTS: Test[] = [
         prefix: ".",
         suffix: "",
         modifier: "?",
-        pattern: "[^\\/#\\?]+?",
+        pattern: "(?:(?!\\.)[^\\/#\\?])+?",
       },
     ],
     [
       ["/route", ["/route", "route", undefined]],
       ["/route.json", ["/route.json", "route", "json"]],
-      ["/route.json.html", ["/route.json.html", "route", "json.html"]],
+      ["/route.json.html", ["/route.json.html", "route.json", "html"]],
     ],
     [
       [{ test: "route" }, "/route"],
@@ -1490,13 +1491,13 @@ const TESTS: Test[] = [
         prefix: ".",
         suffix: "",
         modifier: "?",
-        pattern: "[^\\/#\\?]+?",
+        pattern: "(?:(?!\\.)[^\\/#\\?])+?",
       },
     ],
     [
       ["/route", ["/route", "route", undefined]],
       ["/route.json", ["/route.json", "route", "json"]],
-      ["/route.json.html", ["/route.json.html", "route", "json.html"]],
+      ["/route.json.html", ["/route.json.html", "route.json", "html"]],
     ],
     [
       [{ test: "route" }, "/route"],
@@ -1638,7 +1639,7 @@ const TESTS: Test[] = [
       ")",
     ],
     [["/route(\\123\\)", ["/route(\\123\\)", "123\\"]]],
-    [],
+    [[["123\\"], "/route(\\123\\)"]],
   ],
   [
     "{/login}?",
@@ -1927,7 +1928,7 @@ const TESTS: Test[] = [
       ["/icon-240.png", ["/icon-240.png", "240"]],
       ["/apple-icon-240.png", ["/apple-icon-240.png", "240"]],
     ],
-    [],
+    [[{ res: "240" }, "/icon-240.png"]],
   ],
 
   /**
@@ -1968,8 +1969,11 @@ const TESTS: Test[] = [
       },
       "(test)/bar",
     ],
-    [],
-    [],
+    [
+      ["/foo(test)/bar", ["/foo(test)/bar", "foo"]],
+      ["/another/bar", null],
+    ],
+    [[{ foo: "foo" }, "/foo(test)/bar"]],
   ],
   [
     "/:remote([\\w-.]+)/:user([\\w-]+)",
@@ -2080,7 +2084,7 @@ const TESTS: Test[] = [
         prefix: "",
         suffix: "",
         modifier: "?",
-        pattern: "[^\\/#\\?]+?",
+        pattern: "(?:(?!\\()[^\\/#\\?])+?",
       },
       ")",
     ],
@@ -2286,7 +2290,7 @@ const TESTS: Test[] = [
         prefix: ".",
         suffix: "",
         modifier: "",
-        pattern: "[^\\/#\\?]+?",
+        pattern: "(?:(?!\\.)[^\\/#\\?])+?",
       },
     ],
     [
@@ -2352,14 +2356,14 @@ const TESTS: Test[] = [
     [
       {
         name: "foo",
-        pattern: "[^\\/#\\?]+?",
+        pattern: "(?:(?!\\$)[^\\/#\\?])+?",
         prefix: "$",
         suffix: "",
         modifier: "",
       },
       {
         name: "bar",
-        pattern: "[^\\/#\\?]+?",
+        pattern: "(?:(?!\\$)[^\\/#\\?])+?",
         prefix: "$",
         suffix: "",
         modifier: "?",
@@ -2388,14 +2392,14 @@ const TESTS: Test[] = [
       },
       {
         name: "attr2",
-        pattern: "[^\\/#\\?]+?",
+        pattern: "(?:(?!-)[^\\/#\\?])+?",
         prefix: "-",
         suffix: "",
         modifier: "?",
       },
       {
         name: "attr3",
-        pattern: "[^\\/#\\?]+?",
+        pattern: "(?:(?!-)[^\\/#\\?])+?",
         prefix: "-",
         suffix: "",
         modifier: "?",
@@ -2593,39 +2597,6 @@ const TESTS: Test[] = [
       [{ foo: "#" }, null],
     ],
   ],
-  /**
-   * https://github.com/pillarjs/path-to-regexp/issues/260
-   */
-  [
-    ":name*",
-    undefined,
-    [
-      {
-        name: "name",
-        prefix: "",
-        suffix: "",
-        modifier: "*",
-        pattern: "[^\\/#\\?]+?",
-      },
-    ],
-    [["foobar", ["foobar", "foobar"]]],
-    [[{ name: "foobar" }, "foobar"]],
-  ],
-  [
-    ":name+",
-    undefined,
-    [
-      {
-        name: "name",
-        prefix: "",
-        suffix: "",
-        modifier: "+",
-        pattern: "[^\\/#\\?]+?",
-      },
-    ],
-    [["foobar", ["foobar", "foobar"]]],
-    [[{ name: "foobar" }, "foobar"]],
-  ],
 ];
 
 /**
@@ -2713,14 +2684,14 @@ if (semver.gte(process.version, "10.0.0")) {
         ],
       ],
       [],
-    ]
+    ],
   );
 }
 
 /**
  * Dynamically generate the entire test suite.
  */
-describe("path-to-regexp", function () {
+describe("path-to-regexp", () => {
   const TEST_PATH = "/user/:id";
 
   const TEST_PARAM = {
@@ -2731,8 +2702,8 @@ describe("path-to-regexp", function () {
     pattern: "[^\\/#\\?]+?",
   };
 
-  describe("arguments", function () {
-    it("should work without different call combinations", function () {
+  describe("arguments", () => {
+    it("should work without different call combinations", () => {
       pathToRegexp.pathToRegexp("/test");
       pathToRegexp.pathToRegexp("/test", []);
       pathToRegexp.pathToRegexp("/test", undefined, {});
@@ -2746,7 +2717,7 @@ describe("path-to-regexp", function () {
       pathToRegexp.pathToRegexp(["/a", "/b"], undefined, {});
     });
 
-    it("should accept an array of keys as the second argument", function () {
+    it("should accept an array of keys as the second argument", () => {
       const keys: pathToRegexp.Key[] = [];
       const re = pathToRegexp.pathToRegexp(TEST_PATH, keys, { end: false });
 
@@ -2754,184 +2725,190 @@ describe("path-to-regexp", function () {
       expect(exec(re, "/user/123/show")).toEqual(["/user/123", "123"]);
     });
 
-    it("should throw on non-capturing pattern", function () {
-      expect(function () {
+    it("should throw on non-capturing pattern", () => {
+      expect(() => {
         pathToRegexp.pathToRegexp("/:foo(?:\\d+(\\.\\d+)?)");
       }).toThrow(new TypeError('Pattern cannot start with "?" at 6'));
     });
 
-    it("should throw on nested capturing group", function () {
-      expect(function () {
+    it("should throw on nested capturing group", () => {
+      expect(() => {
         pathToRegexp.pathToRegexp("/:foo(\\d+(\\.\\d+)?)");
       }).toThrow(new TypeError("Capturing groups are not allowed at 9"));
     });
 
-    it("should throw on unbalanced pattern", function () {
-      expect(function () {
+    it("should throw on unbalanced pattern", () => {
+      expect(() => {
         pathToRegexp.pathToRegexp("/:foo(abc");
       }).toThrow(new TypeError("Unbalanced pattern at 5"));
     });
 
-    it("should throw on missing pattern", function () {
-      expect(function () {
+    it("should throw on missing pattern", () => {
+      expect(() => {
         pathToRegexp.pathToRegexp("/:foo()");
       }).toThrow(new TypeError("Missing pattern at 5"));
     });
 
-    it("should throw on missing name", function () {
-      expect(function () {
+    it("should throw on missing name", () => {
+      expect(() => {
         pathToRegexp.pathToRegexp("/:(test)");
       }).toThrow(new TypeError("Missing parameter name at 1"));
     });
 
-    it("should throw on nested groups", function () {
-      expect(function () {
+    it("should throw on nested groups", () => {
+      expect(() => {
         pathToRegexp.pathToRegexp("/{a{b:foo}}");
       }).toThrow(new TypeError("Unexpected OPEN at 3, expected CLOSE"));
     });
 
-    it("should throw on misplaced modifier", function () {
-      expect(function () {
+    it("should throw on misplaced modifier", () => {
+      expect(() => {
         pathToRegexp.pathToRegexp("/foo?");
       }).toThrow(new TypeError("Unexpected MODIFIER at 4, expected END"));
     });
+
+    it("should throw on parameters without text between them", () => {
+      expect(() => {
+        pathToRegexp.pathToRegexp("/:x:y");
+      }).toThrow(
+        new TypeError(
+          `Must have text between two parameters, missing text after "x"`,
+        ),
+      );
+    });
+
+    it("should throw on unrepeatable params", () => {
+      expect(() => {
+        pathToRegexp.pathToRegexp("/foo:x*");
+      }).toThrow(
+        new TypeError(`Can not repeat "x" without a prefix and suffix`),
+      );
+    });
   });
 
-  describe("tokens", function () {
+  describe("tokens", () => {
     const tokens = pathToRegexp.parse(TEST_PATH);
 
-    it("should expose method to compile tokens to regexp", function () {
+    it("should expose method to compile tokens to regexp", () => {
       const re = pathToRegexp.tokensToRegexp(tokens);
 
       expect(exec(re, "/user/123")).toEqual(["/user/123", "123"]);
     });
 
-    it("should expose method to compile tokens to a path function", function () {
+    it("should expose method to compile tokens to a path function", () => {
       const fn = pathToRegexp.tokensToFunction(tokens);
 
       expect(fn({ id: 123 })).toEqual("/user/123");
     });
   });
 
-  describe("rules", function () {
-    TESTS.forEach(function (test) {
-      const [path, opts, tokens, matchCases, compileCases] = test;
-
-      describe(util.inspect(path), function () {
+  describe("rules", () => {
+    TESTS.forEach(([path, opts, tokens, matchCases, compileCases]) => {
+      describe(util.inspect(path), () => {
         const keys: pathToRegexp.Key[] = [];
         const re = pathToRegexp.pathToRegexp(path, keys, opts);
 
         // Parsing and compiling is only supported with string input.
         if (typeof path === "string") {
-          it("should parse", function () {
+          it("should parse", () => {
             expect(pathToRegexp.parse(path, opts)).toEqual(tokens);
           });
 
-          describe("compile", function () {
-            compileCases.forEach(function (io) {
-              const [params, result, options] = io;
+          describe("compile", () => {
+            compileCases.forEach(([params, result, options]) => {
               const toPath = pathToRegexp.compile(path, {
                 ...opts,
                 ...options,
               });
 
               if (result !== null) {
-                it("should compile using " + util.inspect(params), function () {
+                it("should compile using " + util.inspect(params), () => {
                   expect(toPath(params)).toEqual(result);
                 });
               } else {
-                it(
-                  "should not compile using " + util.inspect(params),
-                  function () {
-                    expect(function () {
-                      toPath(params);
-                    }).toThrow(TypeError);
-                  }
-                );
+                it("should not compile using " + util.inspect(params), () => {
+                  expect(() => {
+                    toPath(params);
+                  }).toThrow(TypeError);
+                });
               }
             });
           });
         } else {
-          it("should parse keys", function () {
+          it("should parse keys", () => {
             expect(keys).toEqual(
-              tokens.filter(function (token) {
-                return typeof token !== "string";
-              })
+              tokens.filter((token) => typeof token !== "string"),
             );
           });
         }
 
-        describe(
-          "match" + (opts ? " using " + util.inspect(opts) : ""),
-          function () {
-            matchCases.forEach(function (io) {
-              const [pathname, matches, params, options] = io;
-              const message = `should ${
-                matches ? "" : "not "
-              }match ${util.inspect(pathname)}`;
+        describe("match" + (opts ? " using " + util.inspect(opts) : ""), () => {
+          matchCases.forEach(([pathname, matches, params, options]) => {
+            const message = `should ${
+              matches ? "" : "not "
+            }match ${util.inspect(pathname)}`;
 
-              it(message, function () {
-                expect(exec(re, pathname)).toEqual(matches);
-              });
-
-              if (typeof path === "string" && params !== undefined) {
-                const match = pathToRegexp.match(path, options);
-
-                it(message + " params", function () {
-                  expect(match(pathname)).toEqual(params);
-                });
-              }
+            it(message, () => {
+              expect(exec(re, pathname)).toEqual(matches);
             });
-          }
-        );
+
+            if (typeof path === "string" && params !== undefined) {
+              const match = pathToRegexp.match(path, options);
+
+              it(message + " params", () => {
+                expect(match(pathname)).toEqual(params);
+              });
+            }
+          });
+        });
       });
     });
   });
 
-  describe("compile errors", function () {
-    it("should throw when a required param is undefined", function () {
+  describe("compile errors", () => {
+    it("should throw when a required param is undefined", () => {
       const toPath = pathToRegexp.compile("/a/:b/c");
 
-      expect(function () {
+      expect(() => {
         toPath();
       }).toThrow(new TypeError('Expected "b" to be a string'));
     });
 
-    it("should throw when it does not match the pattern", function () {
+    it("should throw when it does not match the pattern", () => {
       const toPath = pathToRegexp.compile("/:foo(\\d+)");
 
-      expect(function () {
+      expect(() => {
         toPath({ foo: "abc" });
       }).toThrow(
-        new TypeError('Expected "foo" to match "\\d+", but got "abc"')
+        new TypeError('Expected "foo" to match "\\d+", but got "abc"'),
       );
     });
 
-    it("should throw when expecting a repeated value", function () {
+    it("should throw when expecting a repeated value", () => {
       const toPath = pathToRegexp.compile("/:foo+");
 
-      expect(function () {
+      expect(() => {
         toPath({ foo: [] });
       }).toThrow(new TypeError('Expected "foo" to not be empty'));
     });
 
-    it("should throw when not expecting a repeated value", function () {
+    it("should throw when not expecting a repeated value", () => {
       const toPath = pathToRegexp.compile("/:foo");
 
-      expect(function () {
+      expect(() => {
         toPath({ foo: [] });
       }).toThrow(
-        new TypeError('Expected "foo" to not repeat, but got an array')
+        new TypeError('Expected "foo" to not repeat, but got an array'),
       );
     });
 
-    it("should throw when repeated value does not match", function () {
+    it("should throw when repeated value does not match", () => {
       const toPath = pathToRegexp.compile("/:foo(\\d+)+");
 
-      expect(function () {
+      expect(() => {
         toPath({ foo: [1, 2, 3, "a"] });
       }).toThrow(
-        new TypeError('Expected all "foo" to match "\\d+", but got "a"')
+        new TypeError('Expected all "foo" to match "\\d+", but got "a"'),
       );
     });
   });
